//========================================================================================
//
//	File:				FWListBx.h
//	Release Version:	$ ODF 1 $
//
//	Copyright:			(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================


#ifndef FWLISTBX_H
#define FWLISTBX_H

//-----------------------------------------------------------------------------
// Imported Classes
//-----------------------------------------------------------------------------

#ifndef FWFONT_H
#include "FWFont.h"
#endif

#ifndef FWVIEW_H
#include "FWView.h"
#endif

#ifndef FWSTRING_H
#include "FWString.h"
#endif

#ifndef FWNOTIFN_H
#include "FWNotifn.h"
#endif

#ifndef FWNOTIFR_H
#include "FWNotifr.h"
#endif

#ifndef FWNOTDEF_H
#include "FWNotDef.h"
#endif

class FW_CMouseEvent;
class FW_CActivateEvent;
class FW_CVirtualKeyEvent;
class FW_CViewContext;

//-----------------------------------------------------------------------------
// FW_CListBox
//-----------------------------------------------------------------------------
// Simple 1 column list box displaying string items
//
// Because the current Mac implementation uses the native List manager there are 
// a few limitations to FW_CListBox objects:
// - no more than 32K of data (bad performances even before 32K)
// - string items limited to 255 chars
// - no custom drawing of items.  You can only change the font.

class FW_CListBox: public FW_CView, public FW_MNotifier
{
public:
	FW_DECLARE_CLASS
	FW_DECLARE_AUTO(FW_CListBox)

	FW_CListBox(Environment* ev, 
				FW_CSuperView* container,
				const FW_CRect& bounds,
				ODID id,
				int	numItems,				 			// initial number of items
				FW_Boolean scrollVert = TRUE,			// vertical scroll-bar
				const FW_CFont&	font = FW_kNormalFont,
				FW_Message msg = FW_kListBoxDoubleClickedMsg,
				FW_Boolean singleSelection = TRUE, 		// only 1 selected item at a time
				FW_Boolean useFocusFrame = TRUE,	 	// draw border for keyboard focus
				FW_Boolean useClientData = FALSE); 		// extra 32bit value for each item
						
	FW_CListBox(Environment* ev);
				
	~FW_CListBox();
	
	// -------- New API ---------
	
	// Setting/Getting selected item(s)
	virtual void 		SelectOneItem(Environment* ev, short index, FW_Boolean selected = true);
	
	FW_Boolean 			GetSelectedState(Environment* ev, short index) const;
	short 				GetSelectedItem(Environment* ev) const;
	
	short 				GetSelectionCount(Environment* ev) const;
	short 				GetSelectedItems(Environment* ev, short maxItems, short* indexArray) const;

	// Setting value to existing items
	virtual void 		SetStringItem(Environment* ev, short index, const FW_CString& str);
	virtual void 		SetStringItem(Environment* ev, short index, const FW_CString& str,
														void* clientData);

	// Adding new items
	virtual void		AddStringItem(Environment* ev, const FW_CString& str, 
														short index = 0);
	virtual void		AddStringItem(Environment* ev, const FW_CString& str, 
														void* clientData, short index = 0);

	// Deleting items
	virtual void 		DeleteItems(Environment* ev, short index, short count = 1);

	// Getting items
	short 				GetCountItems(Environment* ev) const;
	FW_Boolean			GetStringItem(Environment* ev, short index, FW_CString& str) const;
	FW_Boolean			GetStringItem(Environment* ev, short index, FW_CString& str, 
														void** clientData) const;
	FW_Boolean			GetClientDataItem(Environment* ev, short index, void** clientData) const;

	// Utilities
	FW_Message			GetDoubleClickedMsg(Environment*) const;
	FW_Boolean			GetDrawingMode(Environment* ev) const;
	virtual void		SetDrawingMode(Environment* ev, FW_Boolean draw);
	virtual void 		ScrollIntoView(Environment* ev, short index);

	FW_PlatformHandle	GetPlatformListHandle(Environment* ev) const;

	// ----- inherited API -----

	// from FW_MEventHandle
	virtual FW_Boolean 	DoMouseDown(Environment* ev, const FW_CMouseEvent& theMouseEvent);
	virtual void		DoActivateEvent(Environment *ev, const FW_CActivateEvent& theActivateEvent);
	virtual FW_Boolean 	WantsToBeTarget(Environment* ev);
	virtual FW_Boolean 	DoVirtualKey(Environment* ev, const FW_CVirtualKeyEvent& event);
	virtual void 		ActivateTarget (Environment * ev, FW_Boolean tabSelection);
	virtual void 		DeactivateTarget (Environment * ev);

	// from FW_CView
	virtual void 		Draw (Environment* ev, ODFacet* facet, ODShape* invalidShape);
	virtual void 		SetVisible(Environment* ev, FW_Boolean visible, FW_Boolean propagate);
	virtual FW_Boolean 	AdjustCursor(Environment *ev, ODFacet* odFacet, const FW_CPoint& where);

	// Archiving 
	static void*		Create(FW_CReadableStream& stream, FW_ClassTypeConstant type);
	static void			Destroy(void* object, FW_ClassTypeConstant type);
	virtual void		Flatten(Environment* ev, FW_CWritableStream& stream) const;
	virtual void		InitializeFromStream(Environment* ev, FW_CReadableStream& stream);
	
	// ----- private API -----
private:
	FW_Boolean 			CheckIndex(Environment* ev, short index) const;	
	void 				PrivSelectOneItem(Environment* ev, short index, FW_Boolean selected);
	void				PrivScrollIntoView(short index);

#ifdef FW_BUILD_MAC
	short 				PrivGetSelectedItems(short* count, short maxItems = 0, short* array = NULL) const;
	void 				InitMacListBox(Environment* ev, const FW_CRect& bounds, int numRows, 
										FW_Boolean scrollVert, const FW_CFont& font);
	void 				SetupListBox(FW_CViewContext& vc, FW_Boolean mouseDownEvent = false, 
										FW_CPlatformPoint* qdWhere = NULL);
	void 				MacSetVisible(Environment* ev, FW_Boolean activate);
	void 				MacSetItem(short index, const FW_CString& str, 
								FW_Boolean hasClientData = FALSE, void* clientData = NULL);
#endif

	// ----- private Data -----
private:
	FW_Message			fDoubleClickMsg;	
	FW_Boolean			fSingleSelection;
	FW_Boolean			fUseClientData;		
	FW_Boolean			fDrawingMode;
	
#ifdef FW_BUILD_MAC
	FW_PlatformHandle	fMacListHdl;		
	FW_Boolean			fMacFocusFrame;		
	FW_Boolean			fMacFocusFrameOn;
	short				fMacFontId;	
	short				fMacFontStyle;	
	short				fMacFontSize;		
#endif
};

//========================================================================================
//	CLASS FW_CListBoxNotification
//========================================================================================

class FW_CListBoxNotification : public FW_CNotification
{
public:
	FW_DECLARE_CLASS

	FW_CListBoxNotification(FW_CListBox* listbox);
	FW_CListBoxNotification(const FW_CListBoxNotification& other);
	virtual			~FW_CListBoxNotification();

	FW_CListBoxNotification&	operator=(const FW_CListBoxNotification& other);
	
	FW_CListBox*		GetListBox(Environment* ev) const;
	
private:
	FW_CListBox*		fListBox;
};

//========================================================================================
//  In-lines
//========================================================================================

inline	FW_PlatformHandle	FW_CListBox::GetPlatformListHandle(Environment*) const
{
#ifdef FW_BUILD_MAC
	return fMacListHdl;
#endif
}

inline	FW_Boolean	FW_CListBox::GetDrawingMode(Environment*) const
{
	return fDrawingMode;
}

inline	FW_Message	FW_CListBox::GetDoubleClickedMsg(Environment*) const
{
	return fDoubleClickMsg;
}

inline	FW_CListBox* FW_CListBoxNotification::GetListBox(Environment*) const
{
	return fListBox;
}


#endif // FWLISBX_H


