//========================================================================================
//
//	File:				FWClustr.cpp
//	Release Version:	$ ODF 1 $
//
//	Copyright:	(c) 1993 - 1996 by Apple Computer, Inc., all rights reserved.
//
//========================================================================================

#include "FWFrameW.hpp"

#ifndef FWAROPER_H
#include "FWArOper.h"
#endif

#ifndef FWCLUSTR_H
#include "FWClustr.h"
#endif

#ifndef FWBUTTON_H
#include "FWButton.h"
#endif

#ifndef FWFRAME_H
#include "FWFrame.h"
#endif

#ifndef FWCONTXT_H
#include "FWContxt.h"
#endif

#ifndef FWNOTDEF_H
#include "FWNotDef.h"
#endif

//========================================================================================
// File scope definitions
//========================================================================================

#ifdef FW_BUILD_MAC
#pragma segment fwgadgts
#endif

//========================================================================================
//	CLASS FW_CRadioCluster
//========================================================================================

FW_DEFINE_AUTO(FW_CRadioCluster)
FW_DEFINE_CLASS_M0(FW_CRadioCluster)

// This class is archivable, but we provide the archiving implementation in a separate
// translation unit in order to enable deadstripping of the archiving-related code
// in parts that do not use archiving with this class.

//----------------------------------------------------------------------------------------
// FW_CRadioCluster::FW_CRadioCluster
//----------------------------------------------------------------------------------------

FW_CRadioCluster::FW_CRadioCluster(Environment* ev) :
	FW_MReceiver(),
	fButtonOn(0)
{
FW_UNUSED(ev);
	FW_END_CONSTRUCTOR
}

//----------------------------------------------------------------------------------------
// FW_CRadioCluster::~FW_CRadioCluster
//----------------------------------------------------------------------------------------

FW_CRadioCluster::~FW_CRadioCluster()
{
	FW_START_DESTRUCTOR
}

//----------------------------------------------------------------------------------------
// FW_CRadioCluster::AddRadio
//----------------------------------------------------------------------------------------
 
void FW_CRadioCluster::AddRadio(Environment* ev, FW_CButton* radio)
{
	FW_ASSERT(radio != NULL && radio->GetButtonKind(ev) == FW_kRadioButton);

	// This works only if the radio button uses the defaul button message
	// Otherwise the RadioCluster will not catch the notification
	AddInterest(FW_CInterest(radio, FW_kRadioClusterMsg));

	// Use the first button added to the cluster as the default one
	if (!fButtonOn)
	{
		fButtonOn = radio;
		radio->SetValue(ev, 1);
	}
	else if (radio->GetValue(ev) != 0)
	{
		fButtonOn->SetValue(ev, 0);
		fButtonOn = radio;
	}
}

//----------------------------------------------------------------------------------------
// FW_CRadioCluster::RemoveRadio
//----------------------------------------------------------------------------------------

void FW_CRadioCluster::RemoveRadio(Environment* ev, FW_CButton* radio)
{
FW_UNUSED(ev);
	RemoveInterest(FW_CInterest(radio, FW_kRadioClusterMsg));
}

//----------------------------------------------------------------------------------------
// FW_CRadioCluster::HandleNotification
//----------------------------------------------------------------------------------------

void FW_CRadioCluster::HandleNotification(Environment* ev, const FW_CNotification& notification)
{
	switch (notification.GetMessage())
	{
		case FW_kRadioClusterMsg:
		{
			const FW_CControlNotification& controlNotification = 
													(FW_CControlNotification&) notification;
			FW_CButton* radioClicked = (FW_CButton*)controlNotification.GetControl(ev);
			
			// Toggle buttons
			
			FW_CReceiverInterestIterator ite(this);
			for (FW_CInterest* i = ite.First(); ite.IsNotComplete(); i = ite.Next())
			{
				FW_CButton* radio = FW_DYNAMIC_CAST(FW_CButton, i->GetNotifier());
				FW_ASSERT(radio);
		
				if (radio != radioClicked && radio->GetValue(ev) == 1)
					radio->SetValue(ev, 0);
			}			
			fButtonOn = radioClicked;
		}
		break;
		
		case FW_kNotifierDeletedMsg:
		{
			// Delete radio cluster when the last radio button is being deleted.
			// (we make sure that there is only 1 notifier left in the interest list)
			FW_CReceiverInterestIterator ite(this);
			FW_CInterest* i = ite.First();
			FW_ASSERT(i);
			FW_MNotifier* notifier = i->GetNotifier();
			FW_Boolean deleteIt = TRUE;
			for (; ite.IsNotComplete(); i = ite.Next())
			{
				if (notifier != i->GetNotifier()) {
					deleteIt = FALSE;
					break;
				}
			}
			if (deleteIt) delete this;
		}
		break;	
		
		default:
			FW_ASSERT("FW_CRadioCluster can't handle this message");
			break;
	}
}

//----------------------------------------------------------------------------------------
//	FW_CRadioCluster::Flatten
//----------------------------------------------------------------------------------------

void FW_CRadioCluster::Flatten(Environment* ev, FW_CWritableStream& stream) const
{
	// If all radio buttons have been removed at runtime the radio-cluster was deleted too
	
	FW_CSuperView* superView = fButtonOn->GetSuperView(ev);
	FW_WRITE_DYNAMIC_OBJECT(stream, superView, FW_CSuperView);

	// [LSD] todo: add code to retrieve the list of radio buttons connected	
	short numRadioButtons = 0;  
	long dummy = 0;
	
	stream << dummy, numRadioButtons;
/*
	FW_TOrderedCollectionIterator<FW_CInterest> ite(&fInterestList);
	short numRadioButtons = ite.Count();  
	long dummy = 0;
	
	stream << dummy, numRadioButtons;

	FW_CNotifier* nofifier;
	FW_CButton* button;
	for (anInterest = ite.First(); ite.IsNotComplete(); anInterest = ite.Next())
	{
		notifier = anInterest->GetNotifier();
		button = FW_DYNAMIC_CAST(FW_CButton, notifier);
		FW_ASSERT(button != 0);
		
		stream << button->GetViewId(ev);	
*/
}

//----------------------------------------------------------------------------------------
//	FW_CRadioCluster::InitializeFromStream
//----------------------------------------------------------------------------------------

void FW_CRadioCluster::InitializeFromStream(Environment* ev, FW_CReadableStream& stream)
{
	// Read the parent view of the radio buttons 
	FW_CSuperView* parentView;	
	FW_READ_DYNAMIC_OBJECT(stream, &parentView, FW_CSuperView);
	
	long dummy;
	short numRadioButtons;
	stream >> dummy;		// [LSD] dummy field to work around odfrc bug
	stream >> numRadioButtons;
	
	FW_ASSERT(numRadioButtons > 0);
	
	for (short i = 1; i <= numRadioButtons; i++)
	{
		ODID	radioId;
		stream >> radioId;
		
		FW_CView* view = parentView->FindViewById(ev, radioId);
		FW_CButton* button = FW_DYNAMIC_CAST(FW_CButton, view);
		FW_ASSERT(button != NULL);
		
		AddRadio(ev, button);
	}	
}


